/*   This file is part of Xsltop.
*
*    Xsltop is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation.
*
*    Xsltop is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY.  See the
*    GNU General Public License for more details.
*
*    See <http://www.gnu.org/licenses/>.
*/
package xsltop.gui;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.xml.transform.TransformerException;

import xsltop.Xsltop;
import xsltop.dico.Dico;
import xsltop.dico.Labels;
import xsltop.util.FunctionalLogger;
import xsltop.util.StreamHelper;
import xsltop.xml.NSCxtImpl;
import xsltop.xml.XmlBaseEngine;
import xsltop.xml.XPathHelper;
import xsltop.xml.XsltopException;
import xsltop.xml.XsltopTransformer;
/**
 * It manages the GUI.
 * @author philippe voncken
 *
 */
public class GuiManager {
	private FunctionalLogger logger = null;
	/** the xml editor (visualizer). */
	private XmlEditor xmleditor;
	/** The XML DOM. */
	private XmlBaseEngine xmldom = null;
	/** the xsl editor. */
	private XslEditor xsleditor;
	/** The XSL DOM. */
	private XmlBaseEngine xsldom;
	/** Main Window. */
	private JFrame frame;
	/** The tabbedpan. */
	private JTabbedPane tabbedpan;
	/** The buttons to translate. */
	private Map<String, AbstractButton> buttons = null;
	/** The dictionnary to easily translate. */
	private Dico dico;
	private JTextPane xmloutput;
	private boolean namespaceaware = true;
	/**
	 * private default constructor.
	 *
	 */
	private GuiManager() {		
		dico = Dico.getInstance(Xsltop.DEFAULT_LANG);
		buttons = new HashMap<String, AbstractButton>();
	}
	/**
	 * Constructor by initialization.
	 * @param frame the main window
	 */
	public GuiManager(JFrame frame) {
		this();
		this.frame = frame;
	}
	/**
	 * Update the label text.
	 *
	 */
	private void refreshLabels() {
		for (Map.Entry<String, AbstractButton> e : buttons.entrySet()){			
			e.getValue().setText(dico.getLabel(e.getKey()));
		}
		tabbedpan.setTitleAt(0, dico.getLabel(Labels.LABEL_XML));
		tabbedpan.setTitleAt(1,	dico.getLabel(Labels.LABEL_XPATH));
		tabbedpan.setTitleAt(2,	dico.getLabel(Labels.LABEL_XSL));
		tabbedpan.setTitleAt(3,	dico.getLabel(Labels.LABEL_XMLOUTPUT));
		logger.debug("refresh label ok");
	}
	/**
	 * Perform the open xml file button.
	 */
	public void openXmlFile() {				
		File fichier = getFileToOpen();
		if (fichier != null) {	    	
	    	xmleditor.setFile(fichier);
	    	try {
	    		String xmlcontent = StreamHelper.fileToString(
	    				new BufferedInputStream(new FileInputStream(fichier))
	    				);
	    		performNamespace(xmlcontent);
	    		xmldom = new XmlBaseEngine(
	    				new BufferedInputStream(new FileInputStream(fichier)), 
	    				false, 
	    				namespaceaware
	    				);
	    		this.xmleditor.setText(xmldom.exportToString());
	    		resetXmlTitle();
	    		logger.info(fichier.getAbsolutePath() + " " 
	    				+ getDico().getLabel(Labels.MSG_LOADED));
	    		this.tabbedpan.setSelectedIndex(0);
	    	} catch (XsltopException exception) {
	    		logger.error(exception.getMessage());
	    	}catch (FileNotFoundException exception) {
	    		logger.error(exception.getMessage());
	    	} catch (IOException exception) {
	    		logger.error(exception.getMessage());
	    	}	    		    	
		}				
	}
	/**
	 * To use when final user click on open XSL file button.
	 */
	public void openXslFile() {
		File fichier = getFileToOpen();
		if (fichier != null) {	    	
	    	xsleditor.setFile(fichier);
	    	try {
	    		String xslcontent = StreamHelper.fileToString(
	    				new BufferedInputStream(new FileInputStream(fichier))
	    				);
	    		performNamespace(xslcontent);
	    		xsldom = new XmlBaseEngine(
	    				new BufferedInputStream(new FileInputStream(fichier)), 
	    				false, 
	    				namespaceaware
	    				);
	    		this.xsleditor.setText(xsldom.exportToString());
	    		resetXslTitle();
	    		logger.info(fichier.getAbsolutePath() + " " 
	    				+ getDico().getLabel(Labels.MSG_LOADED));
	    		this.tabbedpan.setSelectedIndex(2);
	    	} catch (XsltopException exception) {
	    		logger.error(exception.getMessage());
	    	}catch (FileNotFoundException exception) {
	    		logger.error(exception.getMessage());
	    	} catch (IOException exception) {
	    		logger.error(exception.getMessage());
	    	}	    		    	
		}
	}
	private void performNamespace(String xmlcontent) {
		Map<String, String> namespaces = StreamHelper.findNamespace(xmlcontent);
		XPathHelper.setNamespaceContext(
				new NSCxtImpl(namespaces)
				);
	}
	/**
	 * Perform the open xml file button.
	 */
	private File getFileToOpen() {
		Config config = Config.getInstance();
		JFileChooser dialogue = new JFileChooser(config.getFilePath());		
		File fichier = null;	    
	    int type = dialogue.showOpenDialog(frame);	    
		if (type == JFileChooser.APPROVE_OPTION) {
	    	fichier = dialogue.getSelectedFile();	    		    		    	
		}
		return fichier;
	}
	/**
	 * To use when final user click on Ok button to request an xpath.
	 * @param xpath expression
	 * @return a List of String which represent the result of the request
	 */
	public List<String> performXPath(String xpath) {		
		List<String> list = new ArrayList<String>();
		if (xpath.equals("")) {
			logger.info(getDico().getLabel(Labels.MSG_XPATHEMPTY));
		} else {
			try {
				if (xmldom == null && xmleditor.isModified()) {
					saveXMLas();
				}
				if (xmldom != null) {
					logger.info(xpath + " " 
							+ getDico().getLabel(Labels.MSG_PERFORM));
					list = this.xmldom.evaluateXPath(xpath);
					logger.info(xpath + " " 
							+ getDico().getLabel(Labels.MSG_XPATHOK));
				}
			} catch (XsltopException exception) {
				logger.error(exception.getMessage());			
			}
		}
		return list;
	}
	void changeXML() {		
		String title = this.tabbedpan.getTitleAt(0);
		if (!title.endsWith(" *")) {
			this.tabbedpan.setTitleAt(0, title + " *");
			this.xmleditor.setModified(true);
			getLogger().debug(
					getDico().getLabel(Labels.MSG_XMLCHANGED)
					);
		}		
	}
	void changeXSL() {		
		String title = this.tabbedpan.getTitleAt(2);
		if (!title.endsWith(" *")) {
			this.tabbedpan.setTitleAt(2, title + " *");
			this.xsleditor.setModified(true);
			logger.debug(getDico().getLabel(Labels.MSG_XSLCHANGED));
		}		
	}
	/**
	 * Save the DOM XML to a file.
	 */
	public void saveXML() {		
		if (xmleditor.isModified()) {
			if (xmleditor.getFile() == null) {
				saveXMLas();
			} else {
				try {
					loadXmlToDom();
					injectDomToXmlEditor();
					saveXmlToFile(xmleditor.getFile());
				} catch (XsltopException exception) {
					logger.error(getDico().getLabel(Labels.MSG_XMLERROR));
				}				
			}			
		} else {
			logger.info(getDico().getLabel(Labels.MSG_XMLNOCHANGED));
		}
	}
	/**
	 * Save the XML DOM in a file which will specified by user.
	 */
	public void saveXMLas() {
		try {
			if (xmleditor.isModified()) {
				loadXmlToDom();
				injectDomToXmlEditor();
				logger.debug("xml " + getDico().getLabel(Labels.MSG_LOADED));
			}
			File file = getFileToSave();
			if (file != null) {			  
				xmleditor.setFile(file);
				saveXmlToFile(xmleditor.getFile());			  
			}		
		} catch (XsltopException exception) {
			logger.info(getDico().getLabel(Labels.MSG_XMLERROR));
		}
	}
	/**
	 * Save the XSL DOM to a file.
	 */
	public void saveXSL() {
		if (xsleditor.isModified()) {
			if (xsleditor.getFile() == null) {
				saveXSLas();
			} else {
				try {
					loadXslToDom();
					injectDomToXslEditor();
					saveXslToFile(xsleditor.getFile());
				} catch (XsltopException exception) {
					logger.error(getDico().getLabel(Labels.MSG_XSLERROR));
				}				
			}			
		} else {
			logger.info(getDico().getLabel(Labels.MSG_XSLNOCHANGED));
		}
	}
	/**
	 * Save XSL DOM to the file which will specified by user.
	 */
	public void saveXSLas() {
		try {
			if (xsleditor.isModified()) {
				loadXslToDom();
				injectDomToXslEditor();
				logger.debug("xsl " + getDico().getLabel(Labels.MSG_LOADED));
			}
			File file = getFileToSave();
			if (file != null) {			  
				xsleditor.setFile(file);
				saveXslToFile(xsleditor.getFile());			  
			}
		} catch (XsltopException exception) {
			logger.info(getDico().getLabel(Labels.MSG_XSLERROR));
		}
	}
	/**
	 * Show a dialog to user. User can selected a file path to save a file.
	 * @return a File selected by user
	 */
	private File getFileToSave() {
		Config config = Config.getInstance();
		JFileChooser dialogue = new JFileChooser(config.getFilePath());		
		File fichier = null;	    
	    int type = dialogue.showSaveDialog(frame);	    
		if (type == JFileChooser.APPROVE_OPTION) {
	    	fichier = dialogue.getSelectedFile();	    		    		    	
		}
		return fichier;
	}
	private void saveXmlToFile(File xmlfile2) {
		try {
			if (xmldom != null) {
				xmldom.exportToFile(xmlfile2);
				logger.info(xmlfile2.getAbsolutePath() + " " 
    				+ getDico().getLabel(Labels.MSG_SAVED));
				resetXmlTitle();
			} else {
				logger.info(getDico().getLabel(Labels.MSG_NODOM));
			}
    	} catch (XsltopException exception) {
    		logger.error(exception.getMessage());
    	}
		
	}
	private void saveXslToFile(File xslfile2) {
		try {
			if (xsldom != null) {
				xsldom.exportToFile(xslfile2);
				logger.info(xslfile2.getAbsolutePath() + " " 
    				+ getDico().getLabel(Labels.MSG_SAVED));
				resetXslTitle();
			} else {
				logger.info(getDico().getLabel(Labels.MSG_NODOM));
			}
    	} catch (XsltopException exception) {
    		logger.error(exception.getMessage());
    	}
		
	}
	/**
	 * Charge le dom.
	 *
	 */
	private void loadXmlToDom() throws XsltopException {
		try {
		InputStream stream = new BufferedInputStream(
				new ByteArrayInputStream(
						this.xmleditor.getText().getBytes("utf-8")
					)
				);
		this.xmldom = new XmlBaseEngine(stream, false, namespaceaware);
		Map<String, String> namespaces = StreamHelper.findNamespace(
				this.xmleditor.getText()
				);
		XPathHelper.setNamespaceContext(
				new NSCxtImpl(namespaces)
				);
		} catch (XsltopException exception) {
			throw exception;
		} catch (UnsupportedEncodingException exception) {			
			throw new XsltopException(exception);
		}
	}
	/**
	 * Charge le dom.
	 *
	 */
	private void loadXslToDom() throws XsltopException {
		try {
		InputStream stream = new BufferedInputStream(
				new ByteArrayInputStream(
						this.xsleditor.getText().getBytes("utf-8")
					)
				);
		this.xsldom = new XmlBaseEngine(stream, false, namespaceaware);
		Map<String, String> namespaces = StreamHelper.findNamespace(
				this.xsleditor.getText()
				);
		XPathHelper.setNamespaceContext(
				new NSCxtImpl(namespaces)
				);
		} catch (XsltopException exception) {
			throw exception;
		} catch (UnsupportedEncodingException exception) {			
			throw new XsltopException(exception);
		}
	}
	/**
	 * Transforme le DOM en string et rafraichi la vue XML.
	 *
	 */
	private void injectDomToXmlEditor() {
		try {
			if(xmldom != null) {
				this.xmleditor.setText(this.xmldom.exportToString());
			} else {
				logger.error(getDico().getLabel(Labels.MSG_NODOM));
			}
		} catch (XsltopException exception) {
			logger.error(getDico().getLabel(Labels.MSG_XMLERROR) 
				+ " Exception:" + exception.getMessage());
		}
	}
	/**
	 * Transforme le DOM en string et rafraichi la vue XML.
	 *
	 */
	private void injectDomToXslEditor() {
		try {
			if(xsldom != null) {
				this.xsleditor.setText(this.xsldom.exportToString());
			} else {
				logger.error(getDico().getLabel(Labels.MSG_NODOM));
			}
		} catch (XsltopException exception) {
			logger.error(getDico().getLabel(Labels.MSG_XMLERROR) 
				+ " Exception:" + exception.getMessage());
		}
	}
	private void resetTitle(int index) {
		String title = this.tabbedpan.getTitleAt(index);
		if (title.endsWith(" *")) {
			String newtitle = title.substring(0, title.length()-2);
			this.tabbedpan.setTitleAt(index, newtitle);
		}	
	}
	private void resetXmlTitle() {
		resetTitle(0);
		this.xmleditor.setModified(false);
	}
	private void resetXslTitle() {
		resetTitle(2);
		this.xsleditor.setModified(false);
	}
	/**
	 * setter to the dictionnary(language) selection.
	 * @param lang french, english or many if developper are motived :)
	 */
	public void setDico(String lang) {
		dico = Dico.getInstance(lang);
		refreshLabels();
	}
	/**
	 * dico getter
	 * @return the dico
	 */
	public Dico getDico() {
		return dico;
	}
	/**
	 * Add a button pointer to translate the label when user change the language
	 * parameter of the xsltop application.
	 * @param key xpath expression to find a label in the dico
	 * @param button to manage the language translation
	 */
	public void addAbstractButton(String key, AbstractButton button) {
		this.buttons.put(key, button);
	}
	/**
	 * To manage the console.
	 * @param consoletext
	 */
	public void setConsole(JTextPane consoletext) {
		logger = new FunctionalLogger(consoletext);
		
	}
	/**
	 * To obtain access to the console.
	 * @return a logger which log in the GUI console only
	 */
	public FunctionalLogger getLogger() {
		return logger;
	}
	/**
	 * @param editor to set
	 */
	void setXmlEditor(XmlEditor editor) {
		this.xmleditor = editor;
	}
	/**
	 * 
	 * @param tabbedpan to set
	 */
	public void setTabbedPane(final JTabbedPane tabbedpan) {
		this.tabbedpan = tabbedpan;
		final GuiManager manager = this;
		this.tabbedpan.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				if (3 == tabbedpan.getSelectedIndex()) {
					try {
						manager.transform();
					} catch (TransformerException e) {
						logger.error(dico.getLabel(Labels.MSG_XSLERROR) 
								+ ": " + e.getMessage());
					}
				}				
			}
		});
	}
	/**
	 * 
	 * @param xslEditor to set
	 */
	public void setXslEditor(XslEditor xslEditor) {
		this.xsleditor = xslEditor;
		
	}
	/**
	 * 
	 * @param xmloutput to set
	 */
	public void setXmlOutput(JTextPane xmloutput) {
		this.xmloutput = xmloutput;
	}
	/**
	 * Transofm the XML DOM with XSL and display the result to XMLOUTPUT.
	 * @throws TransformerException 
	 */
	public void transform() throws TransformerException {
		String out;
		if (this.xmldom != null && this.xsldom != null) {
			out = XsltopTransformer.transform(this.xmldom.getDocument(), 
				this.xsldom.getDocument());
			logger.info(getDico().getLabel(Labels.MSG_XSLT));
		} else {
			out = "";
			logger.error(dico.getLabel(Labels.MSG_XSLERROR));
		}
		this.xmloutput.setText(out);
	}
	/**
	 * To activate or desactivate the namespace in the DOM parser.
	 * @param namespaceaware to set
	 */
	public void setNamespaceaware(boolean namespaceaware) {
		this.namespaceaware = namespaceaware;
		Config.getInstance().setNamespaceaware(
				Boolean.toString(namespaceaware)
				);
	}
}
